@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright CEA/DAM/DIF (2010, 2011)
+# Copyright CEA/DAM/DIF (2010, 2011, 2012)
 #  Contributor: Stephane THIELL <stephane.thiell@cea.fr>
 #
 # This file is part of the ClusterShell library.
@@ -30,8 +30,6 @@
 #
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
-#
-# $Id: Clubak.py 507 2011-06-07 22:33:01Z st-cea $
 
 """
 format dsh/pdsh-like output for humans and more
@@ -44,9 +42,9 @@ from itertools import imap
 import sys
 
 from ClusterShell.MsgTree import MsgTree, MODE_DEFER, MODE_TRACE
-from ClusterShell.NodeSet import STD_GROUP_RESOLVER
+from ClusterShell.NodeSet import RESOLVER_STD_GROUP, NodeSetParseError
 
-from ClusterShell.CLI.Display import Display
+from ClusterShell.CLI.Display import Display, THREE_CHOICES
 from ClusterShell.CLI.Error import GENERIC_ERRORS, handle_generic_error
 from ClusterShell.CLI.OptionParser import OptionParser
 from ClusterShell.CLI.Utils import NodeSet, nodeset_cmp
@@ -73,7 +71,7 @@ def display_tree(tree, disp, out):
         out.write("%s%s\n" % (" " * reldepth, msgline))
         togh = nchildren != 1
 
-def display(tree, disp, gather, trace_mode):
+def display(tree, disp, gather, trace_mode, enable_nodeset_key):
     """nicely display MsgTree instance `tree' content according to
     `disp' Display object and `gather' boolean flag"""
     out = sys.stdout
@@ -82,15 +80,23 @@ def display(tree, disp, gather, trace_mode):
             display_tree(tree, disp, out)
         else:
             if gather:
-                # lambda to create a NodeSet from keys list returned by walk()
-                ns_getter = lambda x: NodeSet.fromlist(x[1])
-                for nodeset in sorted(imap(ns_getter, tree.walk()),
-                                      cmp=nodeset_cmp):
-                    disp.print_gather(nodeset, tree[nodeset[0]])
+                if enable_nodeset_key:
+                    # lambda to create a NodeSet from keys returned by walk()
+                    ns_getter = lambda x: NodeSet.fromlist(x[1])
+                    for nodeset in sorted(imap(ns_getter, tree.walk()),
+                                          cmp=nodeset_cmp):
+                        disp.print_gather(nodeset, tree[nodeset[0]])
+                else:
+                    for msg, key in tree.walk():
+                        disp.print_gather_keys(key, msg)
             else:
-                # nodes are automagically sorted by NodeSet
-                for node in NodeSet.fromlist(tree.keys()):
-                    disp.print_gather(node, tree[node])
+                if enable_nodeset_key:
+                    # nodes are automagically sorted by NodeSet
+                    for node in NodeSet.fromlist(tree.keys()).nsiter():
+                        disp.print_gather(node, tree[str(node)])
+                else:
+                    for key in tree.keys():
+                        disp.print_gather_keys([ key ], tree[key])
     finally:
         out.flush()
 
@@ -99,11 +105,17 @@ def clubak():
 
     # Argument management
     parser = OptionParser("%prog [options]")
-    parser.install_display_options(separator_option=True,
+    parser.install_display_options(verbose_options=True,
+                                   separator_option=True,
                                    dshbak_compat=True,
                                    msgtree_mode=True)
     options = parser.parse_args()[0]
 
+    if options.interpret_keys == THREE_CHOICES[-1]: # auto?
+        enable_nodeset_key = None # AUTO
+    else:
+        enable_nodeset_key = (options.interpret_keys == THREE_CHOICES[1])
+
     # Create new message tree
     if options.trace_mode:
         tree_mode = MODE_TRACE
@@ -112,7 +124,7 @@ def clubak():
     tree = MsgTree(mode=tree_mode)
     fast_mode = options.fast_mode
     if fast_mode:
-        if tree_mode != MODE_DEFER:
+        if tree_mode != MODE_DEFER or options.line_mode:
             parser.error("incompatible tree options")
         preload_msgs = {}
 
@@ -120,14 +132,28 @@ def clubak():
     for line in sys.stdin:
         try:
             linestripped = line.rstrip('\r\n')
-            node, content = linestripped.split(options.separator, 1)
-            node = node.strip()
-            if not node:
+            if options.verbose or options.debug:
+                print "INPUT %s" % linestripped
+            key, content = linestripped.split(options.separator, 1)
+            key = key.strip()
+            if not key:
                 raise ValueError("no node found")
-            if fast_mode:
-                preload_msgs.setdefault(node, []).append(content)
+            if enable_nodeset_key is False: # interpret-keys=never?
+                keyset = [ key ]
             else:
-                tree.add(node, content)
+                try:
+                    keyset = NodeSet(key)
+                except NodeSetParseError:
+                    if enable_nodeset_key: # interpret-keys=always?
+                        raise
+                    enable_nodeset_key = False # auto => switch off
+                    keyset = [ key ]
+            if fast_mode:
+                for node in keyset:
+                    preload_msgs.setdefault(node, []).append(content)
+            else:
+                for node in keyset:
+                    tree.add(node, content)
         except ValueError, ex:
             raise ValueError("%s (\"%s\")" % (ex, linestripped))
 
@@ -137,14 +163,18 @@ def clubak():
         for key, wholemsg in preload_msgs.iteritems():
             tree.add(key, '\n'.join(wholemsg))
 
-    if options.debug:
-        STD_GROUP_RESOLVER.set_verbosity(1)
-        print >> sys.stderr, "clubak: line_mode=%s gather=%s tree_depth=%d" % \
-            (bool(options.line_mode), bool(options.gather), tree._depth())
-
     # Display results
-    disp = Display(options)
-    display(tree, disp, options.gather or disp.regroup, options.trace_mode)
+    try:
+        disp = Display(options)
+        if options.debug:
+            RESOLVER_STD_GROUP.set_verbosity(1)
+            print >> sys.stderr, \
+                "clubak: line_mode=%s gather=%s tree_depth=%d" % \
+                    (bool(options.line_mode), bool(disp.gather), tree._depth())
+        display(tree, disp, disp.gather or disp.regroup, \
+                options.trace_mode, enable_nodeset_key is not False)
+    except ValueError, exc:
+        parser.error("option mismatch (%s)" % exc)
 
 def main():
     """main script function"""
