@@ -1,5 +1,5 @@
 #
-# Copyright CEA/DAM/DIF (2007, 2008, 2009, 2010, 2011)
+# Copyright CEA/DAM/DIF (2007, 2008, 2009, 2010, 2011, 2012)
 #  Contributor: Stephane THIELL <stephane.thiell@cea.fr>
 #
 # This file is part of the ClusterShell library.
@@ -29,15 +29,13 @@
 #
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
-#
-# $Id: Task.py 487 2011-03-15 21:58:07Z st-cea $
 
 """
 ClusterShell Task module.
 
 Simple example of use:
 
->>> from ClusterShell.Task import *
+>>> from ClusterShell.Task import task_self
 >>>  
 >>> # get task associated with calling thread
 ... task = task_self()
@@ -57,9 +55,12 @@ Simple example of use:
 """
 
 from itertools import imap
+import logging
 from operator import itemgetter
+import socket
 import sys
 import threading
+from time import sleep
 import traceback
 
 from ClusterShell.Engine.Engine import EngineAbortException
@@ -70,11 +71,15 @@ from ClusterShell.Engine.Factory import PreferredEngine
 from ClusterShell.Worker.EngineClient import EnginePort
 from ClusterShell.Worker.Ssh import WorkerSsh
 from ClusterShell.Worker.Popen import WorkerPopen
+from ClusterShell.Worker.Tree import WorkerTree
 
 from ClusterShell.Event import EventHandler
 from ClusterShell.MsgTree import MsgTree
 from ClusterShell.NodeSet import NodeSet
 
+from ClusterShell.Topology import TopologyParser, TopologyError
+from ClusterShell.Propagation import PropagationTreeRouter, PropagationChannel
+
 
 class TaskException(Exception):
     """Base task exception."""
@@ -92,6 +97,10 @@ class TaskMsgTreeError(TaskError):
     """Raised when trying to access disabled MsgTree."""
 
 
+def _getshorthostname():
+    """Get short hostname (host name cut at the first dot)"""
+    return socket.gethostname().split('.')[0]
+
 def _task_print_debug(task, s):
     """
     Default task debug printing function. Cannot provide 'print'
@@ -102,37 +111,85 @@ def _task_print_debug(task, s):
 
 class Task(object):
     """
-    Always bound to a thread, the Task class allows you to execute
-    commands in parallel and get their results.
+    The Task class defines an essential ClusterShell object which aims to
+    execute commands in parallel and easily get their results.
 
-    To create a task in a new thread:
+    More precisely, a Task object manages a coordinated (ie. with respect of
+    its current parameters) collection of independent parallel Worker objects.
+    See ClusterShell.Worker.Worker for further details on ClusterShell Workers.
+
+    Always bound to a specific thread, a Task object acts like a "thread
+    singleton". So most of the time, and even more for single-threaded
+    applications, you can get the current task object with the following
+    top-level Task module function:
+        >>> task = task_self()
+
+    However, if you want to create a task in a new thread, use:
         >>> task = Task()
 
-    To create or get the instance of the task associated with the
-    thread object thr (threading.Thread):
+    To create or get the instance of the task associated with the thread
+    object thr (threading.Thread):
         >>> task = Task(thread=thr)
 
-    Add a command to execute locally within task with:
+    To submit a command to execute locally within task, use:
         >>> task.shell("/bin/hostname")
 
-    Add a command to execute to a distant node within task with:
+    To submit a command to execute to some distant nodes in parallel, use:
         >>> task.shell("/bin/hostname", nodes="tiger[1-20]")
 
-    Run task in its associated thread (will block only if the calling
-    thread is the task associated thread):
-        >>> task.resume()
-    """
-    _std_default = {  "stderr"              : False,
-                      "stdout_msgtree"      : True,
-                      "stderr_msgtree"      : True,
-                      "engine"              : 'auto',
-                      "port_qlimit"         : 100 }
+    The previous examples submit commands to execute but do not allow result
+    interaction during their execution. For your program to interact during
+    command execution, it has to define event handlers that will listen for
+    local or remote events. These handlers are based on the EventHandler
+    class, defined in ClusterShell.Event. The following example shows how to
+    submit a command on a cluster with a registered event handler:
+        >>> task.shell("uname -r", nodes="node[1-9]", handler=MyEventHandler())
 
-    _std_info =     { "debug"               : False,
-                      "print_debug"         : _task_print_debug,
-                      "fanout"              : 64,
-                      "connect_timeout"     : 10,
-                      "command_timeout"     : 0 }
+    Run task in its associated thread (will block only if the calling thread is
+    the task associated thread):
+        >>> task.resume()
+    or
+        >>> task.run()
+
+    You can also pass arguments to task.run() to schedule a command exactly
+    like in task.shell(), and run it:
+        >>> task.run("hostname", nodes="tiger[1-20]", handler=MyEventHandler())
+
+    A common need is to set a maximum delay for command execution, especially
+    when the command time is not known. Doing this with ClusterShell Task is
+    very straighforward. To limit the execution time on each node, use the
+    timeout parameter of shell() or run() methods to set a delay in seconds,
+    like:
+        >>> task.run("check_network.sh", nodes="tiger[1-20]", timeout=30)
+
+    You can then either use Task's iter_keys_timeout() method after execution
+    to see on what nodes the command has timed out, or listen for ev_timeout()
+    events in your event handler.
+
+    To get command result, you can either use Task's iter_buffers() method for
+    standard output, iter_errors() for standard error after command execution
+    (common output contents are automatically gathered), or you can listen for
+    ev_read() and ev_error() events in your event handler and get live command
+    output.
+
+    To get command return codes, you can either use Task's iter_retcodes(),
+    node_retcode() and max_retcode() methods after command execution, or
+    listen for ev_hup() events in your event handler.
+    """
+    _std_default = {  "stderr"             : False,
+                      "stdout_msgtree"     : True,
+                      "stderr_msgtree"     : True,
+                      "engine"             : 'auto',
+                      "port_qlimit"        : 100,
+                      "auto_tree"          : False,
+                      "topology_file"      : "/etc/clustershell/topology.conf" }
+
+    _std_info =     { "debug"              : False,
+                      "print_debug"        : _task_print_debug,
+                      "fanout"             : 64,
+                      "grooming_delay"     : 0.25,
+                      "connect_timeout"    : 10,
+                      "command_timeout"    : 0 }
     _tasks = {}
     _taskid_max = 0
     _task_lock = threading.Lock()
@@ -166,7 +223,7 @@ class Task(object):
                     task._dispatch_port.msg_send((f, fargs, kwargs))
                 else:
                     task.info("print_debug")(task, "%s: dropped call: %s" % \
-                                                   (task, fargs))
+                                                   (task, str(fargs)))
             # modify the decorator meta-data for pydoc
             # Note: should be later replaced  by @wraps (functools)
             # as of Python 2.5
@@ -252,12 +309,16 @@ class Task(object):
             self._suspended = False
             self._quit = False
 
+            # Default router
+            self.topology = None
+            self.router = None
+            self.pwrks = {}
+            self.pmwkrs = {}
+
             # STDIN tree
             self._msgtree = None
-
             # STDERR tree
             self._errtree = None
-
             # dict of sources to return codes
             self._d_source_rc = {}
             # dict of return codes to sources
@@ -266,6 +327,8 @@ class Task(object):
             self._max_rc = 0
             # keep timeout'd sources
             self._timeout_sources = set()
+            # allow no-op call to getters before resume()
+            self._reset()
 
             # special engine port for task method dispatching
             self._dispatch_port = EnginePort(self,
@@ -297,13 +360,13 @@ class Task(object):
         task_self(), but do not create any task_self() instance."""
         return self.thread == threading.currentThread()
 
-    def default_excepthook(self, type, value, tb):
+    def default_excepthook(self, exc_type, exc_value, tb):
         """Default excepthook for a newly Task. When an exception is
         raised and uncaught on Task thread, excepthook is called, which
         is default_excepthook by default. Once excepthook overriden,
         you can still call default_excepthook if needed."""
         print >> sys.stderr, 'Exception in thread %s:' % self.thread
-        traceback.print_exception(type, value, tb, file=sys.stderr)
+        traceback.print_exception(exc_type, exc_value, tb, file=sys.stderr)
 
     _excepthook = default_excepthook
 
@@ -348,7 +411,30 @@ class Task(object):
             self._engine.run(timeout)
         finally:
             self._run_lock.release()
-        
+
+    def set_topology(self, topology_file):
+        """Set new propagation topology from provided file."""
+        self.set_default("topology_file", topology_file)
+        self.topology = self._default_topology()
+
+    def _default_topology(self):
+        try:
+            parser = TopologyParser()
+            parser.load(self.default("topology_file"))
+            return parser.tree(_getshorthostname())
+        except TopologyError, exc:
+            logging.getLogger(__name__).exception("_default_topology(): %s", \
+                                                  str(exc))
+            raise
+        return None
+
+    def _default_router(self):
+        if self.router is None:
+            topology = self.topology
+            self.router = PropagationTreeRouter(str(topology.root.nodeset), \
+                                                topology)
+        return self.router
+
     def default(self, default_key, def_val=None):
         """
         Return per-task value for key from the "default" dictionary.
@@ -421,6 +507,8 @@ class Task(object):
             print).
           - "fanout": Max number of registered clients in Engine at a
             time (default: 64).
+          - "grooming_delay": Message maximum end-to-end delay requirement
+            used for traffic grooming, in seconds as float (default: 0.5).
           - "connect_timeout": Time in seconds to wait for connecting to
             remote host before aborting (default: 10).
           - "command_timeout": Time in seconds to wait for a command to
@@ -440,28 +528,25 @@ class Task(object):
 
     def shell(self, command, **kwargs):
         """
-        Schedule a shell command for local or distant parallel
-        execution. This key method in the ClusterShell library creates
-        a local or remote Worker (depending on the presence of the
-        nodes parameter) and immediately schedule it for execution in
-        task runloop. So, if the task is already running (ie. called
-        from an event handler), the command is started immediately,
-        assuming the fanout constraint is met. If the task is not
-        running, the command is not started but scheduled for late
-        execution. See resume() to start task runloop.
+        Schedule a shell command for local or distant parallel execution. This
+        essential method creates a local or remote Worker (depending on the
+        presence of the nodes parameter) and immediately schedules it for
+        execution in task's runloop. So, if the task is already running
+        (ie. called from an event handler), the command is started immediately,
+        assuming current execution contraintes are met (eg. fanout value). If
+        the task is not running, the command is not started but scheduled for
+        late execution. See resume() to start task runloop.
 
-        The following optional parameters are passed to the underlying
-        local or remote Worker constructor:
-          - handler: EventHandler instance to notify (on event) --
-            default is no handler (None)
-          - timeout: command timeout delay expressed in second using
-            a floating point value -- default is unlimited (None)
-          - autoclose: if set to True, the underlying Worker is
-            automatically aborted as soon as all other non-autoclosing
-            task objects (workers, ports, timers) have finished --
-            default is False
-          - stderr: separate stdout/stderr if set to True -- default
-            is False.
+        The following optional parameters are passed to the underlying local
+        or remote Worker constructor:
+          - handler: EventHandler instance to notify (on event) -- default is
+            no handler (None)
+          - timeout: command timeout delay expressed in second using a floating
+            point value -- default is unlimited (None)
+          - autoclose: if set to True, the underlying Worker is automatically
+            aborted as soon as all other non-autoclosing task objects (workers,
+            ports, timers) have finished -- default is False
+          - stderr: separate stdout/stderr if set to True -- default is False.
 
         Local usage::
             task.shell(command [, key=key] [, handler=handler]
@@ -471,7 +556,7 @@ class Task(object):
         Distant usage::
             task.shell(command, nodes=nodeset [, handler=handler]
                   [, timeout=secs], [, autoclose=enable_autoclose]
-                  [, strderr=enable_stderr])
+                  [, strderr=enable_stderr], [tree=None|False|True])
 
         Example:
 
@@ -482,22 +567,34 @@ class Task(object):
 
         handler = kwargs.get("handler", None)
         timeo = kwargs.get("timeout", None)
-        ac = kwargs.get("autoclose", False)
+        autoclose = kwargs.get("autoclose", False)
         stderr = kwargs.get("stderr", self.default("stderr"))
 
         if kwargs.get("nodes", None):
             assert kwargs.get("key", None) is None, \
                     "'key' argument not supported for distant command"
 
-            # create ssh-based worker
-            worker = WorkerSsh(NodeSet(kwargs["nodes"]), command=command,
-                               handler=handler, stderr=stderr, timeout=timeo,
-                               autoclose=ac)
+            tree = kwargs.get("tree")
+            if tree and self.topology is None:
+                raise TaskError("tree mode required for distant shell command" \
+                                " with unknown topology!")
+            if tree is None: # means auto
+                tree = self.default("auto_tree") and (self.topology is not None)
+            if tree:
+                # create tree of ssh worker
+                worker = WorkerTree(NodeSet(kwargs["nodes"]), command=command,
+                                    handler=handler, stderr=stderr,
+                                    timeout=timeo, autoclose=autoclose)
+            else:
+                # create ssh-based worker
+                worker = WorkerSsh(NodeSet(kwargs["nodes"]), command=command,
+                                   handler=handler, stderr=stderr,
+                                   timeout=timeo, autoclose=autoclose)
         else:
             # create (local) worker
             worker = WorkerPopen(command, key=kwargs.get("key", None),
                                  handler=handler, stderr=stderr,
-                                 timeout=timeo, autoclose=ac)
+                                 timeout=timeo, autoclose=autoclose)
 
         # schedule worker for execution in this task
         self.schedule(worker)
@@ -631,10 +728,11 @@ class Task(object):
             self._engine.add(client)
 
     def _resume_thread(self):
-        """Resume called from another thread."""
+        """Resume task - called from another thread."""
         self._suspend_cond.notify_all()
 
     def _resume(self):
+        """Resume task - called from self thread."""
         assert self.thread == threading.currentThread()
         try:
             try:
@@ -642,8 +740,8 @@ class Task(object):
                 self._run(self.timeout)
             except EngineTimeoutException:
                 raise TimeoutError()
-            except EngineAbortException, e:
-                self._terminate(e.kill)
+            except EngineAbortException, exc:
+                self._terminate(exc.kill)
             except EngineAlreadyRunningError:
                 raise AlreadyRunningError("task engine is already running")
         finally:
@@ -655,13 +753,19 @@ class Task(object):
 
     def resume(self, timeout=0):
         """
-        Resume task. If task is task_self(), workers are executed in
-        the calling thread so this method will block until workers have
-        finished. This is always the case for a single-threaded
-        application (eg. which doesn't create other Task() instance
-        than task_self()). Otherwise, the current thread doesn't block.
-        In that case, you may then want to call task_wait() to wait for
-        completion.
+        Resume task. If task is task_self(), workers are executed in the
+        calling thread so this method will block until all (non-autoclosing)
+        workers have finished. This is always the case for a single-threaded
+        application (eg. which doesn't create other Task() instance than
+        task_self()). Otherwise, the current thread doesn't block. In that
+        case, you may then want to call task_wait() to wait for completion.
+
+        Warning: the timeout parameter can be used to set an hard limit of
+        task execution time (in seconds). In that case, a TimeoutError
+        exception is raised if this delay is reached. Its value is 0 by
+        default, which means no task time limit (TimeoutError is never
+        raised). In order to set a maximum delay for individual command
+        execution, you should use Task.shell()'s timeout parameter instead.
         """
         # If you change options here, check Task.run() compatibility.
 
@@ -688,6 +792,15 @@ class Task(object):
         >>> task.shell("hostname", nodes="foo")
         >>> task.shell("hostname", nodes="bar")
         >>> task.run()
+
+        When used with a command, you can set a maximum delay of individual
+        command execution with the help of the timeout parameter (see
+        Task.shell's parameters). You can then listen for ev_timeout() events
+        in your Worker event handlers, or use num_timeout() or
+        iter_keys_timeout() afterwards.
+        But, when used as an alias to Task.resume(), the timeout parameter
+        sets an hard limit of task execution time. In that case, a TimeoutError
+        exception is raised if this delay is reached.
         """
         worker = None
         timeout = 0
@@ -713,6 +826,7 @@ class Task(object):
 
     @tasksyncmethod()
     def _suspend_wait(self):
+        """Suspend request received."""
         assert task_self() == self
         # atomically set suspend state
         self._suspend_lock.acquire()
@@ -756,6 +870,7 @@ class Task(object):
 
     @tasksyncmethod()
     def _abort(self, kill=False):
+        """Abort request received."""
         assert task_self() == self
         # raise an EngineAbortException when task is running
         self._engine.abort(kill)
@@ -787,7 +902,7 @@ class Task(object):
             # invalidate dispatch port
             self._dispatch_port = None
         # clear engine
-        self._engine.clear()
+        self._engine.clear(clear_ports=kill)
 
         # clear result objects
         self._reset()
@@ -872,11 +987,7 @@ class Task(object):
         self._d_source_rc[source] = rc
 
         # store source by rc
-        e = self._d_rc_sources.get(rc)
-        if e is None:
-            self._d_rc_sources[rc] = set([source])
-        else:
-            self._d_rc_sources[rc].add(source)
+        self._d_rc_sources.setdefault(rc, set()).add(source)
         
         # update max rc
         if rc > self._max_rc:
@@ -1150,44 +1261,92 @@ class Task(object):
             if thread != from_thread:
                 task.join()
 
+    def pchannel(self, gateway, metaworker): #gw_invoke_cmd):
+        """Get propagation channel for gateway (create one if needed)"""
+        # create channel if needed
+        if gateway not in self.pwrks:
+            chan = PropagationChannel(self)
+            # invoke gateway
+            timeout = 0
+            worker = self.shell(metaworker.invoke_gateway, nodes=gateway,
+                                handler=chan, timeout=timeout, tree=False)
+            self.pwrks[gateway] = worker
+        else:
+            worker = self.pwrks[gateway]
+            chan = worker.eh
+        
+        if metaworker not in self.pmwkrs:
+            mw = self.pmwkrs[metaworker] = set()
+        else:
+            mw = self.pmwkrs[metaworker]
+        if worker not in mw:
+            #print >>sys.stderr, "pchannel++"
+            worker.metarefcnt += 1
+            mw.add(worker)
+        return chan
+
+    def _pchannel_release(self, metaworker):
+        """Release propagation channel"""
+        if metaworker in self.pmwkrs:
+            for worker in self.pmwkrs[metaworker]:
+                #print >>sys.stderr, "pchannel_release2 %s" % worker
+                worker.metarefcnt -= 1
+                if worker.metarefcnt == 0:
+                    #print >>sys.stderr, "worker abort"
+                    worker.eh._close()
+                    #worker.abort()
+            
 
 def task_self():
     """
-    Get the Task instance bound to the current thread. This function
-    provided as a convenience is available in the top-level
-    ClusterShell.Task package namespace.
+    Return the current Task object, corresponding to the caller's thread of
+    control (a Task object is always bound to a specific thread). This function
+    provided as a convenience is available in the top-level ClusterShell.Task
+    package namespace.
     """
     return Task(thread=threading.currentThread())
 
 def task_wait():
     """
-    Suspend execution of the calling thread until all tasks terminate,
-    unless all tasks have already terminated. This function is provided
-    as a convenience and is available in the top-level
-    ClusterShell.Task package namespace.
+    Suspend execution of the calling thread until all tasks terminate, unless
+    all tasks have already terminated. This function is provided as a
+    convenience and is available in the top-level ClusterShell.Task package
+    namespace.
     """
     Task.wait(threading.currentThread())
 
 def task_terminate():
     """
-    Destroy the Task instance bound to the current thread. A next call
-    to task_self() will create a new Task object. This function provided
-    as a convenience is available in the top-level ClusterShell.Task
-    package namespace.
+    Destroy the Task instance bound to the current thread. A next call to
+    task_self() will create a new Task object. This function provided as a
+    convenience is available in the top-level ClusterShell.Task package
+    namespace.
     """
     task_self().abort(kill=True)
 
 def task_cleanup():
     """
-    Cleanup routine to destroy all created tasks. This function
-    provided as a convenience is available in the top-level
-    ClusterShell.Task package namespace.
+    Cleanup routine to destroy all created tasks. This function provided as a
+    convenience is available in the top-level ClusterShell.Task package
+    namespace. This is mainly used for testing purposes and should be avoided
+    otherwise. task_cleanup() may be called from any threads.
     """
-    Task._task_lock.acquire()
-    try:
-        tasks = Task._tasks.copy()
-    finally:
-        Task._task_lock.release()
-    for task in tasks.itervalues():
-        task.abort(kill=True)
-
+    # be sure to return to a clean state (no task at all)
+    while True:
+        Task._task_lock.acquire()
+        try:
+            tasks = Task._tasks.copy()
+            if len(tasks) == 0:
+                break
+        finally:
+            Task._task_lock.release()
+        # send abort to all known tasks (it's needed to retry as we may have
+        # missed the engine notification window (it was just exiting, which is
+        # quite a common case if we didn't task_join() previously), or we may
+        # have lost some task's dispatcher port messages.
+        for task in tasks.itervalues():
+            task.abort(kill=True)
+        # also, for other task than self, task.abort() is async and performed
+        # through an EngineAbortException, so tell the Python scheduler to give
+        # up control to raise this exception (handled by task._terminate())...
+        sleep(0.001)
