@@ -1,11 +1,11 @@
 #!/usr/bin/env python
 # ClusterShell.NodeSet.RangeSet test suite
 # Written by S. Thiell
-# $Id: RangeSetTest.py 512 2011-06-09 16:58:25Z st-cea $
 
 
 """Unit test for RangeSet"""
 
+import binascii
 import copy
 import pickle
 import sys
@@ -13,7 +13,7 @@ import unittest
 
 sys.path.insert(0, '../lib')
 
-from ClusterShell.NodeSet import RangeSet
+from ClusterShell.RangeSet import RangeSet
 
 
 class RangeSetTest(unittest.TestCase):
@@ -41,9 +41,11 @@ class RangeSetTest(unittest.TestCase):
 
     def testStepAdvanced(self):
         """test RangeSet advanced step usages"""
-        self._testRS("1-4/4,2-6/2", "1-2,4,6", 4)
+        self._testRS("1-4/4,2-6/2", "1,2-6/2", 4)   # 1.6 small behavior change
         self._testRS("6-24/6,9-21/6", "6-24/3", 7)
         self._testRS("0-24/2,9-21/2", "0-8/2,9-22,24", 20)
+        self._testRS("0-24/2,9-21/2,100", "0-8/2,9-22,24,100", 21)
+        self._testRS("0-24/2,9-21/2,100-101", "0-8/2,9-22,24,100-101", 22)
         self._testRS("3-21/9,6-24/9,9-27/9", "3-27/3", 9)
         self._testRS("101-121/4,1-225/112", "1,101-121/4,225", 8)
         self._testRS("1-32/3,13-28/9", "1-31/3", 11)
@@ -54,11 +56,13 @@ class RangeSetTest(unittest.TestCase):
         self._testRS("1-16/3,1-16/6", "1-16/3", 6)
         self._testRS("1-16/6,1-16/3", "1-16/3", 6)
         self._testRS("1-16/3,3-19/6", "1,3-4,7,9-10,13,15-16", 9)
-        self._testRS("1-16/3,3-19/4", "1,3-4,7,10-11,13,15-16,19", 10)
+        #self._testRS("1-16/3,3-19/4", "1,3-4,7,10-11,13,15-16,19", 10) # < 1.6
+        self._testRS("1-16/3,3-19/4", "1,3,4-10/3,11-15/2,16,19", 10)   # >= 1.6
         self._testRS("1-17/2,2-18/2", "1-18", 18)
         self._testRS("1-17/2,33-41/2,2-18/2", "1-18,33-41/2", 23)
         self._testRS("1-17/2,33-41/2,2-20/2", "1-18,20,33-41/2", 24)
         self._testRS("1-17/2,33-41/2,2-19/2", "1-18,33-41/2", 23)
+        self._testRS("1968-1970,1972,1975,1978-1981,1984-1989", "1968-1970,1972-1978/3,1979-1981,1984-1989", 15)
 
     def testIntersectSimple(self):
         """test RangeSet with simple intersections of ranges"""
@@ -84,6 +88,10 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(str(r3), "690-700,800")
         self.assertEqual(len(r3), 12)
 
+        r1 = RangeSet()
+        r3 = r1.intersection(r2)
+        self.assertEqual(str(r3), "")
+        self.assertEqual(len(r3), 0)
 
     def testIntersectStep(self):
         """test RangeSet with more intersections of ranges"""
@@ -122,6 +130,35 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(str(r3), "4,7")
         self.assertEqual(len(r3), 2)
 
+        # bounds checking
+        r1 = RangeSet("1-10,39-41,50-60")
+        r2 = RangeSet("1-10,38-39,50-60")
+        r1.difference_update(r2)
+        self.assertEqual(len(r1), 2)
+        self.assertEqual(str(r1), "40-41")
+
+        r1 = RangeSet("1-20,39-41")
+        r2 = RangeSet("1-20,41-42")
+        r1.difference_update(r2)
+        self.assertEqual(len(r1), 2)
+        self.assertEqual(str(r1), "39-40")
+
+        # difference(self) issue
+        r1 = RangeSet("1-20,39-41")
+        r1.difference_update(r1)
+        self.assertEqual(len(r1), 0)
+        self.assertEqual(str(r1), "")
+
+        # strict mode
+        r1 = RangeSet("4,7-33")
+        r2 = RangeSet("8-33")
+        r1.difference_update(r2, strict=True)
+        self.assertEqual(str(r1), "4,7")
+        self.assertEqual(len(r1), 2)
+
+        r3 = RangeSet("4-5")
+        self.assertRaises(KeyError, r1.difference_update, r3, True)
+
     def testSymmetricDifference(self):
         """test RangeSet.symmetric_difference_update()"""
         r1 = RangeSet("4,7-33")
@@ -139,12 +176,73 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(str(r3), "4,7,34")
         self.assertEqual(len(r3), 3)
 
+        r1 = RangeSet("5,7,10-12,33-50")
+        r2 = RangeSet("8-34")
+        r1.symmetric_difference_update(r2)
+        self.assertEqual(str(r1), "5,7-9,13-32,35-50")
+        self.assertEqual(len(r1), 40)
+
+        r1 = RangeSet("8-34")
+        r2 = RangeSet("5,7,10-12,33-50")
+        r1.symmetric_difference_update(r2)
+        self.assertEqual(str(r1), "5,7-9,13-32,35-50")
+        self.assertEqual(len(r1), 40)
+
+        r1 = RangeSet("8-30")
+        r2 = RangeSet("31-40")
+        r1.symmetric_difference_update(r2)
+        self.assertEqual(str(r1), "8-40")
+        self.assertEqual(len(r1), 33)
+
+        r1 = RangeSet("8-30")
+        r2 = RangeSet("8-30")
+        r1.symmetric_difference_update(r2)
+        self.assertEqual(str(r1), "")
+        self.assertEqual(len(r1), 0)
+
+        r1 = RangeSet("8-30")
+        r2 = RangeSet("10-13,31-40")
+        r1.symmetric_difference_update(r2)
+        self.assertEqual(str(r1), "8-9,14-40")
+        self.assertEqual(len(r1), 29)
+
+        r1 = RangeSet("10-13,31-40")
+        r2 = RangeSet("8-30")
+        r1.symmetric_difference_update(r2)
+        self.assertEqual(str(r1), "8-9,14-40")
+        self.assertEqual(len(r1), 29)
+
+        r1 = RangeSet("1,3,5,7")
+        r2 = RangeSet("4-8")
+        r1.symmetric_difference_update(r2)
+        self.assertEqual(str(r1), "1,3-4,6,8")
+        self.assertEqual(len(r1), 5)
+
+        r1 = RangeSet("1-1000")
+        r2 = RangeSet("0-40,60-100/4,300,1000,1002")
+        r1.symmetric_difference_update(r2)
+        self.assertEqual(str(r1), "0,41-59,61-63,65-67,69-71,73-75,77-79,81-83,85-87,89-91,93-95,97-99,101-299,301-999,1002")
+        self.assertEqual(len(r1), 949)
+
+        r1 = RangeSet("25,27,29-31,33-35,41-43,48,50-52,55-60,63,66-68,71-78")
+        r2 = RangeSet("27-30,35,37-39,42,45-48,50,52-54,56,61,67,69-79,81-82")
+        r1.symmetric_difference_update(r2)
+        self.assertEqual(str(r1), "25,28,31,33-34,37-39,41,43,45-47,51,53-55,57-61,63,66,68-70,79,81-82")
+        self.assertEqual(len(r1), 30)
+
+        r1 = RangeSet("986-987,989,991-992,994-995,997,1002-1008,1010-1011,1015-1018,1021")
+        r2 = RangeSet("989-990,992-994,997-1000")
+        r1.symmetric_difference_update(r2)
+        self.assertEqual(str(r1), "986-987,990-991,993,995,998-1000,1002-1008,1010-1011,1015-1018,1021")
+        self.assertEqual(len(r1), 23)
 
     def testSubStep(self):
         """test RangeSet with more sub of ranges (with step)"""
         # case 1 no sub
         r1 = RangeSet("4-34/2", autostep=3)
         r2 = RangeSet("3-33/2", autostep=3)
+        self.assertEqual(r1.autostep, 3)
+        self.assertEqual(r2.autostep, 3)
         r1.difference_update(r2)
         self.assertEqual(str(r1), "4-34/2")
         self.assertEqual(len(r1), 16)
@@ -192,14 +290,14 @@ class RangeSetTest(unittest.TestCase):
         r1 = RangeSet("1-100,102,105-242,800", autostep=3)
         r2 = RangeSet("1-1000/3", autostep=3)
         r1.difference_update(r2)
-        self.assertEqual(str(r1), "2-3,5-6,8-9,11-12,14-15,17-18,20-21,23-24,26-27,29-30,32-33,35-36,38-39,41-42,44-45,47-48,50-51,53-54,56-57,59-60,62-63,65-66,68-69,71-72,74-75,77-78,80-81,83-84,86-87,89-90,92-93,95-96,98-99,102,105,107-108,110-111,113-114,116-117,119-120,122-123,125-126,128-129,131-132,134-135,137-138,140-141,143-144,146-147,149-150,152-153,155-156,158-159,161-162,164-165,167-168,170-171,173-174,176-177,179-180,182-183,185-186,188-189,191-192,194-195,197-198,200-201,203-204,206-207,209-210,212-213,215-216,218-219,221-222,224-225,227-228,230-231,233-234,236-237,239-240,242,800")
+        self.assertEqual(str(r1), "2-3,5-6,8-9,11-12,14-15,17-18,20-21,23-24,26-27,29-30,32-33,35-36,38-39,41-42,44-45,47-48,50-51,53-54,56-57,59-60,62-63,65-66,68-69,71-72,74-75,77-78,80-81,83-84,86-87,89-90,92-93,95-96,98,99-105/3,107-108,110-111,113-114,116-117,119-120,122-123,125-126,128-129,131-132,134-135,137-138,140-141,143-144,146-147,149-150,152-153,155-156,158-159,161-162,164-165,167-168,170-171,173-174,176-177,179-180,182-183,185-186,188-189,191-192,194-195,197-198,200-201,203-204,206-207,209-210,212-213,215-216,218-219,221-222,224-225,227-228,230-231,233-234,236-237,239-240,242,800")
         self.assertEqual(len(r1), 160)
 
-        r1 = RangeSet("1-100000", autostep=3)
-        r2 = RangeSet("2-99999/2", autostep=3)
+        r1 = RangeSet("1-1000", autostep=3)
+        r2 = RangeSet("2-999/2", autostep=3)
         r1.difference_update(r2)
-        self.assertEqual(str(r1), "1-99999/2,100000")
-        self.assertEqual(len(r1), 50001)
+        self.assertEqual(str(r1), "1-999/2,1000")
+        self.assertEqual(len(r1), 501)
 
         r1 = RangeSet("1-100/3,40-60/3", autostep=3)
         r2 = RangeSet("31-61/3", autostep=3)
@@ -219,7 +317,7 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(len(r1), 240)
         self.assert_(99 in r1)
         self.assert_("99" in r1)
-        self.assert_(not "099" in r1)
+        self.assert_("099" in r1)
         self.assertRaises(TypeError, r1.__contains__, object())
         self.assert_(101 not in r1)
         self.assertEqual(len(r1), 240)
@@ -233,17 +331,28 @@ class RangeSetTest(unittest.TestCase):
         self.assert_(101 not in r2)
         r3 = RangeSet("0003-0143,0360-1000")
         self.assert_(360 in r3)
-        self.assert_(not "360" in r3)
+        self.assert_("360" in r3)
         self.assert_("0360" in r3)
         r4 = RangeSet("00-02")
         self.assert_("00" in r4)
         self.assert_(0 in r4)
-        self.assert_(not "0" in r4)
+        self.assert_("0" in r4)
         self.assert_("01" in r4)
         self.assert_(1 in r4)
-        self.assert_(not "1" in r4)
+        self.assert_("1" in r4)
         self.assert_("02" in r4)
         self.assert_(not "03" in r4)
+        #
+        r1 = RangeSet("115-117,130,132,166-170,4780-4999")
+        self.assertEqual(len(r1), 230)
+        r2 = RangeSet("116-117,130,4781-4999")
+        self.assertEqual(len(r2), 222)
+        self.assertTrue(r2 in r1)
+        self.assertFalse(r1 in r2)
+        r2 = RangeSet("5000")
+        self.assertFalse(r2 in r1)
+        r2 = RangeSet("4999")
+        self.assertTrue(r2 in r1)
 
     def testIsSuperSet(self):
         """test RangeSet.issuperset()"""
@@ -251,26 +360,32 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(len(r1), 240)
         r2 = RangeSet("3-98,140-199,800")
         self.assertEqual(len(r2), 157)
-        self.assert_(r1.issuperset(r1))
-        self.assert_(r1.issuperset(r2))
-        self.assert_(r1 >= r1)
-        self.assert_(r1 > r2)
-        self.assert_(not r2 > r1)
+        self.assertTrue(r1.issuperset(r1))
+        self.assertTrue(r1.issuperset(r2))
+        self.assertTrue(r1 >= r1)
+        self.assertTrue(r1 > r2)
+        self.assertFalse(r2 > r1)
         r2 = RangeSet("3-98,140-199,243,800")
         self.assertEqual(len(r2), 158)
-        self.assert_(not r1.issuperset(r2))
-        self.assert_(not r1 > r2)
+        self.assertFalse(r1.issuperset(r2))
+        self.assertFalse(r1 > r2)
 
     def testIsSubSet(self):
         """test RangeSet.issubset()"""
         r1 = RangeSet("1-100,102,105-242,800-900/2")
         r2 = RangeSet("3,800,802,804,888")
-        self.assert_(r2.issubset(r2))
-        self.assert_(r2.issubset(r1))
-        self.assert_(r2 <= r1)
-        self.assert_(r2 < r1)
-        self.assert_(not r1 < r2)
-        self.assert_(not r1 <= r2)
+        self.assertTrue(r2.issubset(r2))
+        self.assertTrue(r2.issubset(r1))
+        self.assertTrue(r2 <= r1)
+        self.assertTrue(r2 < r1)
+        self.assertTrue(r1 > r2)
+        self.assertFalse(r1 < r2)
+        self.assertFalse(r1 <= r2)
+        self.assertFalse(r2 >= r1)
+        # since v1.6, padding is ignored when computing set operations
+        r1 = RangeSet("1-100")
+        r2 = RangeSet("001-100")
+        self.assertTrue(r1.issubset(r2))
 
     def testGetItem(self):
         """test RangeSet.__getitem__()"""
@@ -287,6 +402,8 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(r1[237], 241)
         self.assertEqual(r1[238], 242)
         self.assertEqual(r1[239], 800)
+        self.assertRaises(IndexError, r1.__getitem__, 240)
+        self.assertRaises(IndexError, r1.__getitem__, 241)
         # negative indices
         self.assertEqual(r1[-1], 800)
         self.assertEqual(r1[-240], 1)
@@ -310,7 +427,7 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(r2[31], 102)
         self.assertEqual(r2[32], 104)
         self.assertEqual(r2[33], 106)
-        #self.assertRaises(TypeError, r2['foo'])
+        self.assertRaises(TypeError, r2.__getitem__, "foo")
 
     def testGetSlice(self):
         """test RangeSet.__getitem__() with slice"""
@@ -389,7 +506,6 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(r1[1:7:2], RangeSet("2,9,11"))
 
         # negative indices - sl_step
-        self.assertRaises(IndexError, r1.__getitem__, slice(1, None, -2))
         self.assertEqual(r1[::-2], RangeSet("1,4,10,12"))
         r2 = RangeSet("1-2,4,9,10-13")
         self.assertEqual(r2[::-2], RangeSet("2,9,11,13"))
@@ -416,12 +532,11 @@ class RangeSetTest(unittest.TestCase):
         self.assertRaises(TypeError, r1.__getitem__, slice('foo', 'bar'))
         self.assertRaises(TypeError, r1.__getitem__, slice(1, 3, 'bar'))
 
-        # TODO: timeit?
+        r3 = RangeSet("0-600")
+        self.assertEqual(r3[30:389], RangeSet("30-388"))
         r3 = RangeSet("0-6000")
-        self.assertEqual(r3[300:3890], RangeSet("300-3889"))
-        r3 = RangeSet("0-6000")
-        self.assertEqual(r3[300:3890:2], RangeSet("300-3889/2"))
-        self.assertEqual(r3[300:3890:2], RangeSet("300-3889/2", autostep=2))
+        self.assertEqual(r3[30:389:2], RangeSet("30-389/2"))
+        self.assertEqual(r3[30:389:2], RangeSet("30-389/2", autostep=2))
 
     def testSplit(self):
         """test RangeSet.split()"""
@@ -448,14 +563,40 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(len(r1), 240)
         r1.add(801)
         self.assertEqual(len(r1), 241)
+        self.assertEqual(r1[0], 1)
         self.assertEqual(r1[240], 801)
         r1.add(788)
         self.assertEqual(str(r1), "1-100,102,105-242,788,800-801")
         self.assertEqual(len(r1), 242)
+        self.assertEqual(r1[0], 1)
         self.assertEqual(r1[239], 788)
         self.assertEqual(r1[240], 800)
         r1.add(812)
         self.assertEqual(len(r1), 243)
+        # test forced padding
+        r1 = RangeSet("1-100,102,105-242,800")
+        r1.add(801, pad=3)
+        self.assertEqual(len(r1), 241)
+        self.assertEqual(str(r1), "001-100,102,105-242,800-801")
+        r1.padding = 4
+        self.assertEqual(len(r1), 241)
+        self.assertEqual(str(r1), "0001-0100,0102,0105-0242,0800-0801")
+
+    def testUpdate(self):
+        """test RangeSet.update()"""
+        r1 = RangeSet("1-100,102,105-242,800")
+        self.assertEqual(len(r1), 240)
+        r2 = RangeSet("243-799,1924-1984")
+        self.assertEqual(len(r2), 618)
+        r1.update(r2)
+        self.assertEqual(type(r1), RangeSet)
+        self.assertEqual(r1.padding, None)
+        self.assertEqual(len(r1), 240+618) 
+        self.assertEqual(str(r1), "1-100,102,105-800,1924-1984")
+        r1 = RangeSet("1-100,102,105-242,800")
+        r1.union_update(r2)
+        self.assertEqual(len(r1), 240+618) 
+        self.assertEqual(str(r1), "1-100,102,105-800,1924-1984")
 
     def testUnion(self):
         """test RangeSet.union()"""
@@ -464,6 +605,8 @@ class RangeSetTest(unittest.TestCase):
         r2 = RangeSet("243-799,1924-1984")
         self.assertEqual(len(r2), 618)
         r3 = r1.union(r2)
+        self.assertEqual(type(r3), RangeSet)
+        self.assertEqual(r3.padding, None)
         self.assertEqual(len(r3), 240+618) 
         self.assertEqual(str(r3), "1-100,102,105-800,1924-1984")
         r4 = r1 | r2
@@ -488,7 +631,20 @@ class RangeSetTest(unittest.TestCase):
         self.assertRaises(KeyError, r1.remove, 101)
         # test remove integer-castable type (convenience)
         r1.remove("106")
-        self.assertRaises(KeyError, r1.remove, "foo")
+        # non integer castable cases raise ValueError (documented since 1.6)
+        self.assertRaises(ValueError, r1.remove, "foo")
+
+    def testDiscard(self):
+        """test RangeSet.discard()"""
+        r1 = RangeSet("1-100,102,105-242,800")
+        self.assertEqual(len(r1), 240)
+        r1.discard(100)
+        self.assertEqual(len(r1), 239)
+        self.assertEqual(str(r1), "1-99,102,105-242,800")
+        r1.discard(101)     # should not raise KeyError
+        # test remove integer-castable type (convenience)
+        r1.remove("106")
+        r1.discard("foo")
 
     def testClear(self):
         """test RangeSet.clear()"""
@@ -499,6 +655,25 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(len(r1), 0)
         self.assertEqual(str(r1), "")
     
+    def testConstructorIterate(self):
+        """test RangeSet(iterable) constructor"""
+        # from list
+        rgs = RangeSet([3,5,6,7,8,1])
+        self.assertEqual(str(rgs), "1,3,5-8")
+        self.assertEqual(len(rgs), 6)
+        rgs.add(10)
+        self.assertEqual(str(rgs), "1,3,5-8,10")
+        self.assertEqual(len(rgs), 7)
+        # from set
+        rgs = RangeSet(set([3,5,6,7,8,1]))
+        self.assertEqual(str(rgs), "1,3,5-8")
+        self.assertEqual(len(rgs), 6)
+        # from RangeSet
+        r1 = RangeSet("1,3,5-8")
+        rgs = RangeSet(r1)
+        self.assertEqual(str(rgs), "1,3,5-8")
+        self.assertEqual(len(rgs), 6)
+
     def testFromListConstructor(self):
         """test RangeSet.fromlist() constructor"""
         rgs = RangeSet.fromlist([ "3", "5-8", "1" ])
@@ -507,16 +682,67 @@ class RangeSetTest(unittest.TestCase):
         rgs = RangeSet.fromlist([ RangeSet("3"), RangeSet("5-8"), RangeSet("1") ])
         self.assertEqual(str(rgs), "1,3,5-8")
         self.assertEqual(len(rgs), 6)
+        rgs = RangeSet.fromlist([set([3,5,6,7,8,1])])
+        self.assertEqual(str(rgs), "1,3,5-8")
+        self.assertEqual(len(rgs), 6)
 
-    def testIterate(self):
+    def testFromOneConstructor(self):
+        """test RangeSet.fromone() constructor"""
+        rgs = RangeSet.fromone(42)
+        self.assertEqual(str(rgs), "42")
+        self.assertEqual(len(rgs), 1)
+        # also support slice object (v1.6+)
+        rgs = RangeSet.fromone(slice(42))
+        self.assertEqual(str(rgs), "0-41")
+        self.assertEqual(len(rgs), 42)
+        self.assertRaises(ValueError, RangeSet.fromone, slice(12, None))
+        rgs = RangeSet.fromone(slice(42, 43))
+        self.assertEqual(str(rgs), "42")
+        self.assertEqual(len(rgs), 1)
+        rgs = RangeSet.fromone(slice(42, 48))
+        self.assertEqual(str(rgs), "42-47")
+        self.assertEqual(len(rgs), 6)
+        rgs = RangeSet.fromone(slice(42, 57, 2))
+        self.assertEqual(str(rgs), "42,44,46,48,50,52,54,56")
+        rgs.autostep = 3
+        self.assertEqual(str(rgs), "42-56/2")
+        self.assertEqual(len(rgs), 8)
+
+    def testIterator(self):
         """test RangeSet iterator"""
-        rgs = RangeSet.fromlist([ "11", "3", "5-8", "1", "4" ])
         matches = [ 1, 3, 4, 5, 6, 7, 8, 11 ]
+        rgs = RangeSet.fromlist([ "11", "3", "5-8", "1", "4" ])
         cnt = 0
         for rg in rgs:
+            self.assertEqual(rg, matches[cnt])
+            cnt += 1
+        self.assertEqual(cnt, len(matches))
+        # with padding
+        rgs = RangeSet.fromlist([ "011", "003", "005-008", "001", "004" ])
+        cnt = 0
+        for rg in rgs:
+            self.assertTrue(type(rg) is int)
+            self.assertEqual(rg, matches[cnt])
+            cnt += 1
+        self.assertEqual(cnt, len(matches))
+
+    def testStringIterator(self):
+        """test RangeSet string iterator striter()"""
+        matches = [ 1, 3, 4, 5, 6, 7, 8, 11 ]
+        rgs = RangeSet.fromlist([ "11", "3", "5-8", "1", "4" ])
+        cnt = 0
+        for rg in rgs.striter():
             self.assertEqual(rg, str(matches[cnt]))
             cnt += 1
         self.assertEqual(cnt, len(matches))
+        # with padding
+        rgs = RangeSet.fromlist([ "011", "003", "005-008", "001", "004" ])
+        cnt = 0
+        for rg in rgs.striter():
+            self.assertTrue(type(rg) is str)
+            self.assertEqual(rg, "%0*d" % (3, matches[cnt]))
+            cnt += 1
+        self.assertEqual(cnt, len(matches))
 
     def testBinarySanityCheck(self):
         """test RangeSet binary sanity check"""
@@ -602,8 +828,9 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(len(r1), 112)
         self.assertEqual(str(r1), "1-100,399,401,403,405,407,409,411,413,415,417,419,421")
         # With autostep...
-        r1 = RangeSet(autostep=2)
+        r1 = RangeSet(autostep=3)
         r1.add_range(1, 100, 1)
+        self.assertEqual(r1.autostep, 3)
         self.assertEqual(len(r1), 99)
         self.assertEqual(str(r1), "1-99")
         r1.add_range(40, 101, 1)
@@ -616,6 +843,7 @@ class RangeSetTest(unittest.TestCase):
         r1 = RangeSet("1-30", autostep=2)
         self.assertEqual(len(r1), 30)
         self.assertEqual(str(r1), "1-30")
+        self.assertEqual(r1.autostep, 2)
         r1.add_range(32, 35, 1)
         self.assertEqual(len(r1), 33)
         self.assertEqual(str(r1), "1-30,32-34")
@@ -631,14 +859,15 @@ class RangeSetTest(unittest.TestCase):
         r1.add_range(40, 65, 10)
         self.assertEqual(len(r1), 13)
         self.assertEqual(str(r1), "1,5,9,13,17,21,25,29-31,40,50,60")
-        r1 = RangeSet("1-30", autostep=2)
+        r1 = RangeSet("1-30", autostep=3)
         r1.add_range(40, 65, 10)
+        self.assertEqual(r1.autostep, 3)
         self.assertEqual(len(r1), 33)
-        self.assertEqual(str(r1), "1-30,40-60/10")
+        self.assertEqual(str(r1), "1-29,30-60/10")
         # One
         r1.add_range(103, 104)
         self.assertEqual(len(r1), 34)
-        self.assertEqual(str(r1), "1-30,40-60/10,103")
+        self.assertEqual(str(r1), "1-29,30-60/10,103")
         # Zero
         self.assertRaises(AssertionError, r1.add_range, 103, 103)
 
@@ -647,28 +876,18 @@ class RangeSetTest(unittest.TestCase):
         r1 = RangeSet()
         self.assertEqual(len(r1), 0)
         self.assertEqual(len(list(r1.slices())), 0)
-        # With padding, without autostep
+        # Without autostep
         r1 = RangeSet("1-7/2,8-12,3000-3019")
+        self.assertEqual(r1.autostep, None)
         self.assertEqual(len(r1), 29)
-        self.assertEqual(list(r1.slices()), [(slice(1, 2, 1), 0), \
-            (slice(3, 4, 1), 0), (slice(5, 6, 1), 0), (slice(7, 13, 1), 0), \
-            (slice(3000, 3020, 1), 0)])
-        # With padding, with autostep
+        self.assertEqual(list(r1.slices()), [slice(1, 2, 1), slice(3, 4, 1), \
+            slice(5, 6, 1), slice(7, 13, 1), slice(3000, 3020, 1)])
+        # With autostep
         r1 = RangeSet("1-7/2,8-12,3000-3019", autostep=2)
         self.assertEqual(len(r1), 29)
-        self.assertEqual(list(r1.slices()), [(slice(1, 8, 2), 0), \
-            (slice(8, 13, 1), 0), (slice(3000, 3020, 1), 0)])
-        # Without padding, without autostep
-        r1 = RangeSet("1-7/2,8-12,3000-3019")
-        self.assertEqual(len(r1), 29)
-        self.assertEqual(list(r1.slices(False)), [slice(1, 2, 1), \
-            slice(3, 4, 1), slice(5, 6, 1), slice(7, 13, 1), \
+        self.assertEqual(r1.autostep, 2)
+        self.assertEqual(list(r1.slices()), [slice(1, 8, 2), slice(8, 13, 1), \
             slice(3000, 3020, 1)])
-        # Without padding, with autostep
-        r1 = RangeSet("1-7/2,8-12,3000-3019", autostep=2)
-        self.assertEqual(len(r1), 29)
-        self.assertEqual(list(r1.slices(False)), [slice(1, 8, 2), \
-            slice(8, 13, 1), slice(3000, 3020, 1)])
 
     def testCopy(self):
         """test RangeSet.copy()"""
@@ -691,8 +910,81 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(str(r1), "115-117,130,167-170,4780-4999")
         self.assertEqual(str(r2), "115-118,130,166-170,4780-4999")
 
-    def testPickle(self):
-        """test pickling of RangeSet"""
+    def test_unpickle_v1_3_py24(self):
+        """test RangeSet unpickling (against v1.3/py24)"""
+        rngset = pickle.loads(binascii.a2b_base64("gAIoY0NsdXN0ZXJTaGVsbC5Ob2RlU2V0ClJhbmdlU2V0CnEAb3EBfXECKFUHX2xlbmd0aHEDS2RVCV9hdXRvc3RlcHEER1SySa0llMN9VQdfcmFuZ2VzcQVdcQYoKEsFSwVLAUsAdHEHKEsHS2ZLAUsAdHEIKEtoS2hLAUsAdHEJKEtqS2tLAUsAdHEKZXViLg=="))
+        self.assertEqual(rngset, RangeSet("5,7-102,104,106-107"))
+        self.assertEqual(str(rngset), "5,7-102,104,106-107")
+        self.assertEqual(len(rngset), 100)
+        self.assertEqual(rngset[0], 5)
+        self.assertEqual(rngset[1], 7)
+        self.assertEqual(rngset[-1], 107)
+
+    def test_unpickle_v1_3_py26(self):
+        """test RangeSet unpickling (against v1.3/py26)"""
+        rngset = pickle.loads(binascii.a2b_base64("gAIoY0NsdXN0ZXJTaGVsbC5Ob2RlU2V0ClJhbmdlU2V0CnEAb3EBfXECKFUHX2xlbmd0aHEDS2RVCV9hdXRvc3RlcHEER1SySa0llMN9VQdfcmFuZ2VzcQVdcQYoKEsFSwVLAUsAdHEHKEsHS2ZLAUsAdHEIKEtoS2hLAUsAdHEJKEtqS2tLAUsAdHEKZXViLg=="))
+        self.assertEqual(rngset, RangeSet("5,7-102,104,106-107"))
+        self.assertEqual(str(rngset), "5,7-102,104,106-107")
+        self.assertEqual(len(rngset), 100)
+        self.assertEqual(rngset[0], 5)
+        self.assertEqual(rngset[1], 7)
+        self.assertEqual(rngset[-1], 107)
+
+    # unpickle_v1_4_py24 : unpickling fails as v1.4 does not have slice pickling workaround
+
+    def test_unpickle_v1_4_py26(self):
+        """test RangeSet unpickling (against v1.4/py26)"""
+        rngset = pickle.loads(binascii.a2b_base64("gAIoY0NsdXN0ZXJTaGVsbC5Ob2RlU2V0ClJhbmdlU2V0CnEAb3EBfXEDKFUHX2xlbmd0aHEES2RVCV9hdXRvc3RlcHEFR1SySa0llMN9VQdfcmFuZ2VzcQZdcQcoY19fYnVpbHRpbl9fCnNsaWNlCnEISwVLBksBh3EJUnEKSwCGcQtoCEsHS2dLAYdxDFJxDUsAhnEOaAhLaEtpSwGHcQ9ScRBLAIZxEWgIS2pLbEsBh3ESUnETSwCGcRRlVQhfdmVyc2lvbnEVSwJ1Yi4="))
+        self.assertEqual(rngset, RangeSet("5,7-102,104,106-107"))
+        self.assertEqual(str(rngset), "5,7-102,104,106-107")
+        self.assertEqual(len(rngset), 100)
+        self.assertEqual(rngset[0], 5)
+        self.assertEqual(rngset[1], 7)
+        self.assertEqual(rngset[-1], 107)
+
+    def test_unpickle_v1_5_py24(self):
+        """test RangeSet unpickling (against v1.5/py24)"""
+        rngset = pickle.loads(binascii.a2b_base64("gAIoY0NsdXN0ZXJTaGVsbC5Ob2RlU2V0ClJhbmdlU2V0CnEAb3EBfXEDKFUHX2xlbmd0aHEES2RVCV9hdXRvc3RlcHEFR1SySa0llMN9VQdfcmFuZ2VzcQZdcQcoSwVLBksBh3EISwCGcQlLB0tnSwGHcQpLAIZxC0toS2lLAYdxDEsAhnENS2pLbEsBh3EOSwCGcQ9lVQhfdmVyc2lvbnEQSwJ1Yi4="))
+        self.assertEqual(rngset, RangeSet("5,7-102,104,106-107"))
+        self.assertEqual(str(rngset), "5,7-102,104,106-107")
+        self.assertEqual(len(rngset), 100)
+        self.assertEqual(rngset[0], 5)
+        self.assertEqual(rngset[1], 7)
+        self.assertEqual(rngset[-1], 107)
+
+    def test_unpickle_v1_5_py26(self):
+        """test RangeSet unpickling (against v1.5/py26)"""
+        rngset = pickle.loads(binascii.a2b_base64("gAIoY0NsdXN0ZXJTaGVsbC5Ob2RlU2V0ClJhbmdlU2V0CnEAb3EBfXEDKFUHX2xlbmd0aHEES2RVCV9hdXRvc3RlcHEFR1SySa0llMN9VQdfcmFuZ2VzcQZdcQcoY19fYnVpbHRpbl9fCnNsaWNlCnEISwVLBksBh3EJUnEKSwCGcQtoCEsHS2dLAYdxDFJxDUsAhnEOaAhLaEtpSwGHcQ9ScRBLAIZxEWgIS2pLbEsBh3ESUnETSwCGcRRlVQhfdmVyc2lvbnEVSwJ1Yi4="))
+
+        self.assertEqual(rngset, RangeSet("5,7-102,104,106-107"))
+        self.assertEqual(str(rngset), "5,7-102,104,106-107")
+        self.assertEqual(len(rngset), 100)
+        self.assertEqual(rngset[0], 5)
+        self.assertEqual(rngset[1], 7)
+        self.assertEqual(rngset[-1], 107)
+
+    def test_unpickle_v1_6_py24(self):
+        """test RangeSet unpickling (against v1.6/py24)"""
+        rngset = pickle.loads(binascii.a2b_base64("gAJjQ2x1c3RlclNoZWxsLlJhbmdlU2V0ClJhbmdlU2V0CnEAVRM1LDctMTAyLDEwNCwxMDYtMTA3cQGFcQJScQN9cQQoVQdwYWRkaW5ncQVOVQlfYXV0b3N0ZXBxBkdUskmtJZTDfVUIX3ZlcnNpb25xB0sDdWIu"))
+        self.assertEqual(rngset, RangeSet("5,7-102,104,106-107"))
+        self.assertEqual(str(rngset), "5,7-102,104,106-107")
+        self.assertEqual(len(rngset), 100)
+        self.assertEqual(rngset[0], 5)
+        self.assertEqual(rngset[1], 7)
+        self.assertEqual(rngset[-1], 107)
+
+    def test_unpickle_v1_6_py26(self):
+        """test RangeSet unpickling (against v1.6/py26)"""
+        rngset = pickle.loads(binascii.a2b_base64("gAJjQ2x1c3RlclNoZWxsLlJhbmdlU2V0ClJhbmdlU2V0CnEAVRM1LDctMTAyLDEwNCwxMDYtMTA3cQGFcQJScQN9cQQoVQdwYWRkaW5ncQVOVQlfYXV0b3N0ZXBxBkdUskmtJZTDfVUIX3ZlcnNpb25xB0sDdWIu"))
+        self.assertEqual(rngset, RangeSet("5,7-102,104,106-107"))
+        self.assertEqual(str(rngset), "5,7-102,104,106-107")
+        self.assertEqual(len(rngset), 100)
+        self.assertEqual(rngset[0], 5)
+        self.assertEqual(rngset[1], 7)
+        self.assertEqual(rngset[-1], 107)
+
+    def test_pickle_current(self):
+        """test RangeSet pickling (current version)"""
         dump = pickle.dumps(RangeSet("1-100"))
         self.assertNotEqual(dump, None)
         rngset = pickle.loads(dump)
@@ -702,7 +994,93 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(rngset[1], 2)
         self.assertEqual(rngset[-1], 100)
 
+    def testIntersectionLength(self):
+        """test RangeSet intersection/length"""
+        r1 = RangeSet("115-117,130,166-170,4780-4999")
+        self.assertEqual(len(r1), 229)
+        r2 = RangeSet("116-117,130,4781-4999")
+        self.assertEqual(len(r2), 222)
+        res = r1.intersection(r2)
+        self.assertEqual(len(res), 222)
+        r1 = RangeSet("115-200")
+        self.assertEqual(len(r1), 86)
+        r2 = RangeSet("116-117,119,123-131,133,149,199")
+        self.assertEqual(len(r2), 15)
+        res = r1.intersection(r2)
+        self.assertEqual(len(res), 15)
+        # StopIteration test
+        r1 = RangeSet("115-117,130,166-170,4780-4999,5003")
+        self.assertEqual(len(r1), 230)
+        r2 = RangeSet("116-117,130,4781-4999")
+        self.assertEqual(len(r2), 222)
+        res = r1.intersection(r2)
+        self.assertEqual(len(res), 222)
+        # StopIteration test2
+        r1 = RangeSet("130,166-170,4780-4999")
+        self.assertEqual(len(r1), 226)
+        r2 = RangeSet("116-117")
+        self.assertEqual(len(r2), 2)
+        res = r1.intersection(r2)
+        self.assertEqual(len(res), 0)
+
+    def testFolding(self):
+        """test RangeSet folding conditions"""
+        r1 = RangeSet("112,114-117,119,121,130,132,134,136,138,139-141,144,147-148", autostep=6)
+        self.assertEqual(str(r1), "112,114-117,119,121,130,132,134,136,138-141,144,147-148")
+        r1.autostep = 5
+        self.assertEqual(str(r1), "112,114-117,119,121,130-138/2,139-141,144,147-148")
+        
+        r1 = RangeSet("1,3-4,6,8")
+        self.assertEqual(str(r1), "1,3-4,6,8")
+        r1 = RangeSet("1,3-4,6,8", autostep=4)
+        self.assertEqual(str(r1), "1,3-4,6,8")
+        r1 = RangeSet("1,3-4,6,8", autostep=2)
+        self.assertEqual(str(r1), "1,3,4-8/2")
+        r1 = RangeSet("1,3-4,6,8", autostep=3)
+        self.assertEqual(str(r1), "1,3,4-8/2")
+
+        # empty set
+        r1 = RangeSet(autostep=3)
+        self.assertEqual(str(r1), "")
+
+    def test_ior(self):
+        """test RangeSet.__ior__()"""
+        r1 = RangeSet("1,3-9,14-21,30-39,42")
+        r2 = RangeSet("2-5,10-32,35,40-41")
+        r1 |= r2
+        self.assertEqual(len(r1), 42)
+        self.assertEqual(str(r1), "1-42")
+
+    def test_iand(self):
+        """test RangeSet.__iand__()"""
+        r1 = RangeSet("1,3-9,14-21,30-39,42")
+        r2 = RangeSet("2-5,10-32,35,40-41")
+        r1 &= r2
+        self.assertEqual(len(r1), 15)
+        self.assertEqual(str(r1), "3-5,14-21,30-32,35")
+
+    def test_ixor(self):
+        """test RangeSet.__ixor__()"""
+        r1 = RangeSet("1,3-9,14-21,30-39,42")
+        r2 = RangeSet("2-5,10-32,35,40-41")
+        r1 ^= r2
+        self.assertEqual(len(r1), 27)
+        self.assertEqual(str(r1), "1-2,6-13,22-29,33-34,36-42")
+
+    def test_isub(self):
+        """test RangeSet.__isub__()"""
+        r1 = RangeSet("1,3-9,14-21,30-39,42")
+        r2 = RangeSet("2-5,10-32,35,40-41")
+        r1 -= r2
+        self.assertEqual(len(r1), 12)
+        self.assertEqual(str(r1), "1,6-9,33-34,36-39,42")
+
+    def test_contiguous(self):
+        r1 = RangeSet("1,3-9,14-21,30-39,42")
+        self.assertEqual(['1', '3-9', '14-21', '30-39', '42'], [str(ns) for ns in r1.contiguous()])
+
 
 if __name__ == '__main__':
     suite = unittest.TestLoader().loadTestsFromTestCase(RangeSetTest)
     unittest.TextTestRunner(verbosity=2).run(suite)
+        
